function [mu,gamma,dmu,k] = bcs_prior(T, m2d, iters, Lanczos_iter, sigma2, gamma_init)
% *************************************************************************
% 
% *** PURPOSE *** 
% Implements generalized versions of SBL for learning sparse
% representations from possibly overcomplete dictionaries.
%
%
% *** USAGE ***
% [mu,dmu,k,gamma] = bcs_prior(T, m2d, iters, Lanczos_iter, sigma2, gamma_init);
%
%
% *** INPUTS ***
% T         = N X L data matrix (undersampled k-space)
% m2d       = undersampling mask
% iters     = maximum number of iterations
% Lanczos iter = no of iterations for diagonalization in matrix inversion
% sigma2    = noise variance
% gamma_init = initial guess for hyperparameters
% 
%
% *** OUTPUTS ***
% mu        = M X L matrix of weight estimates
% dmu       = delta-mu at convergence
% k         = number of iterations used
% gamma     = M X 1 vector of hyperparameter values
%
%
% *************************************************************************
% Written by:  David Wipf, david.wipf@mrsc.ucsf.edu
% Modified by : Berkin Bilgic, berkin@mit.edu
% *************************************************************************
    

% *** Initializations ***
[N L] = size(T);
im_size = size(m2d,1);      % assume square image size
N = sum(m2d(:));
M = im_size^2;

MIN_GAMMA = 1e-16;  
MIN_DMU   = 1e-12;  

if nargin < 3
    iters = 5;
end
if nargin < 4
    Lanczos_iter = N;
end
if nargin < 5
    sigma2 = 1e-6;
end
if nargin < 6
    gamma = ones(M,1);    
else
    gamma = gamma_init + MIN_GAMMA;  % to make sure we are not discarding coefficients prior to iterations 
end

 
keep_list = [1:M]';
m = length(keep_list);
mu = zeros(M,L);
dmu = -1;
k = 0;


% *** Learning loop ***
while (1)
    
    % *** Prune things as hyperparameters go to zero ***
    if (min(gamma) < MIN_GAMMA ) 
		index = find(gamma > MIN_GAMMA);
		gamma = gamma(index);
        
		keep_list = keep_list(index);
        m = length(gamma);
     
        if (m == 0);   break;  end;
    end;
    
    
    % *** Compute new weights ***
    [z,ldT,Q,Tri_alpha,Tri_beta] = diaginv_lanczos_complex(im_size, sigma2, m2d, gamma, keep_list, min(N,Lanczos_iter), 1);
    SIGMA_diag = gamma - z;


    mu_old = mu;  
    x = [];
    for h = 1:L
        x(:,h) = invert_tridiagonal(Tri_beta,Tri_alpha,Tri_beta,Q'*T(:,h));
    end
    
    mu = [];
    for h = 1:L
        temp1 = zeros(im_size);
        temp1(m2d) = Q*x(:,h);
        temp2 = zeros(im_size);
        temp2(keep_list) = gamma;
        temp1 = temp2 .* ifft2(temp1) * im_size;
        mu(:,h) = temp1(keep_list);
    end
    
    gamma_old = gamma;
    gamma = (sum(abs(mu).^2,2)/L) ./ (1-SIGMA_diag./gamma);
      
    
    % *** Check stopping conditions, etc. ***
  	k = k+1;   
    disp(['iters: ',num2str(k),'   num coeffs: ',num2str(m), '   gamma change: ',num2str(max(abs(gamma - gamma_old)))]); 
 
    if (size(mu) == size(mu_old))
        dmu = max(max(abs(mu_old - mu)));
        if (dmu < MIN_DMU)  break;  end;
    end;
   
    gamma_img = zeros(im_size);
    gamma_img(keep_list) = gamma;
    figure(1), imagesc(sqrt(gamma_img)), axis image off, colormap jet, colorbar, title('Standard Deviation'), drawnow   
    
   if (k >= iters);        break;     end;
    
end;


% *** Expand weights, hyperparameters ***
temp = zeros(M,1);
if (m > 0); temp(keep_list,1) = gamma;  end;
gamma = temp;

temp = zeros(M,L);
if (m > 0); temp(keep_list,:) = mu;  end;
mu = temp;
   
return;
