%% load complex-valued Shepp-Logan phantoms

load ATLAS_200x200

% resize images to 128x128 => recon should take ~10 min
img = imresize(img,[128,128],'bicubic');

im_size = 128;

img_full = img(:,:,1);        % fully sampled prior image [Proton density]
img = img(:,:,2:3);           % images to be undersampled [T2 and T1 weighted]

figure(1), imagesc(abs([img(:,:,1),img(:,:,2), img_full]), [0,.7]), axis image off, colormap gray


%% undersampling in kx-ky

R = 8;
pdf = genPDF([im_size,im_size], 5, 1/R, 2, 0.01, 0);
m2d = fftshift(genSampling(pdf,10,1));
figure(2), imagesc(fftshift(m2d)), axis image off, colormap gray, title('Sampling mask')


%% generate undersampled k-space

L = size(img,3);
[k2,k1] = meshgrid(0:im_size-1,0:im_size-1);
fdx = ones(im_size,im_size) - exp(-2*pi*1i*k1/im_size);
fdy = ones(im_size,im_size) - exp(-2*pi*1i*k2/im_size);

zf_disp = [];   Tx = [];    Ty = [];    zfx = [];   zfy = [];   dx_true = [];   dy_true = [];   T = []; zf = [];
for t = 1:L
    T(:,:,t) = fft2(img(:,:,t)) .* m2d;
    temp = T(:,:,t).*fdx / im_size;
    zfx(:,:,t) = ifft2(temp) * im_size;
    dx_true(:,:,t) = ifft2(fft2(img(:,:,t)).*fdx);
    Tx(:,t) = temp(m2d(:));
    
    temp = T(:,:,t).*fdy / im_size;
    zfy(:,:,t) = ifft2(temp) * im_size;
    dy_true(:,:,t) = ifft2(fft2(img(:,:,t)).*fdy);
    Ty(:,t) = temp(m2d(:));
    
    zf(:,:,t) = ifft2(T(:,:,t));
    zf_disp = [zf_disp, zf(:,:,t)];
end

figure(3), imagesc( abs(zf_disp),[0,.7] ), axis image off, colormap gray, title(['Zero filling: ', num2str(100*norm(zf(:)-img(:))/norm(img(:))),' percent RMSE' ]), drawnow


%% run Bayesian CS recon

INITIAL_GAMMA = 2;              % set to 2 to use fully-sampled prior image to initialize iterations
                                % set to 1 to use zero-filling recon for initialization

num_iter = 5;                   % no of EM iterations
Lanczos_iter = sum(m2d(:));     % max no of Lanczos iterations for matrix inversion
sigma2 = 1e-6;                  % noise variance, recon should not be sensitive to this parameter


if INITIAL_GAMMA == 1
    % initial guess from zero-filling recon
    gamma0x = R * sum( abs(zfx).^2, 3 ) / L;    figure(1), imagesc( sqrt(gamma0x) ), axis image off, colormap jet, colorbar, drawnow
    gamma0y = R * sum( abs(zfy).^2, 3 ) / L;    figure(2), imagesc( sqrt(gamma0y) ), axis image off, colormap jet, colorbar, drawnow
else
    if INITIAL_GAMMA == 2
        % initial guess from fully sampled image
        dx_full = ifft2(fft2(img_full).*fdx);
        dy_full = ifft2(fft2(img_full).*fdy);
        gamma0x = abs(dx_full).^2;              figure(1), imagesc( sqrt(gamma0x) ), axis image off, colormap jet, colorbar, drawnow
        gamma0y = abs(dy_full).^2;              figure(2), imagesc( sqrt(gamma0y) ), axis image off, colormap jet, colorbar, drawnow
    end
end



tic;    
    % estimate vertical and horizontal gradients
    [dx,gammax] = bcs_prior(Tx, m2d, num_iter, Lanczos_iter, sigma2, gamma0x(:));
    [dy,gammay] = bcs_prior(Ty, m2d, num_iter, Lanczos_iter, sigma2, gamma0y(:));
toc;
        

im_disp = [];   im_true = [];        
for t = 1:L
    % find the image that matches the reconstructed gradients and the undersampled k-space data 
    im_res(:,:,t) = L2_image_from_edges( T(:,:,t), reshape(dx(:,t), [im_size,im_size]), reshape(dy(:,t), [im_size,im_size]), 0);
    im_disp = [im_disp, im_res(:,:,t)];
    im_true = [im_true, img(:,:,t)];
end

figure(1), imagesc( abs(im_disp),[0,.7] ), axis image off, colormap gray, title(['Lanczos BCS: ', num2str(100*norm(im_res(:)-img(:))/norm(img(:))),' percent RMSE' ]), drawnow
figure(2), imagesc( abs(im_true),[0,.7] ), axis image off, colormap gray, title('Nyquist rate'), drawnow
figure(3), imagesc( abs(im_disp-im_true),[0,.1] ), axis image off, colormap gray, title('Abs error image')


